@@ -1,12 +1,14 @@
 /*
- * Copyright (C) 2022 Guido Günther
+ * Copyright (C) 2023 The Phosh Developers
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ *
+ * Author: Guido Günther <agx@sigxcpu.org>
  *
  * Heavily based on GTK's:
  * gtkemojichooser.c: An Emoji chooser widget
  * Copyright 2017, Red Hat, Inc.
  * Author: Matthias Clasen <mclasen@redhat.com>
- *
- * SPDX-License-Identifier: GPL-3.0-or-later
  */
 
 #include "pos-emoji-picker.h"
@@ -84,6 +86,7 @@ on_backspace_clicked (PosEmojiPicker *self)
   g_signal_emit (self, signals[DELETE_LAST], 0);
 }
 
+
 static void
 scroll_to_section (GtkButton *button,
                    gpointer   data)
@@ -92,15 +95,25 @@ scroll_to_section (GtkButton *button,
   PosEmojiPicker *self;
   GtkAdjustment *adj;
   GtkAllocation alloc = { 0 };
+  double value;
 
   self = POS_EMOJI_PICKER (gtk_widget_get_ancestor (GTK_WIDGET (button), POS_TYPE_EMOJI_PICKER));
 
   adj = gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (self->scrolled_window));
 
   gtk_widget_get_allocation (section->box, &alloc);
-  gtk_adjustment_set_value (adj, alloc.x - BOX_SPACE);
+
+  if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL) {
+    double width = gtk_widget_get_allocated_width (GTK_WIDGET (self));
+    value = alloc.x + alloc.width - width;
+  } else {
+    value = alloc.x - BOX_SPACE;
+  }
+
+  gtk_adjustment_set_value (adj, value);
 }
 
+
 static void
 add_emoji (GtkWidget      *box,
            gboolean        prepend,
@@ -383,7 +396,7 @@ add_emoji (GtkWidget      *box,
 static GBytes *
 get_emoji_data (void)
 {
-  return g_resources_lookup_data ("/sm/puri/phosh/osk-stub/emoji/en.data", 0, NULL);
+  return g_resources_lookup_data ("/mobi/phosh/osk-stub/emoji/en.data", 0, NULL);
 }
 
 static gboolean
@@ -453,7 +466,7 @@ static void
 adj_value_changed (GtkAdjustment *adj, gpointer data)
 {
   PosEmojiPicker *self = data;
-  double value = gtk_adjustment_get_value (adj);
+  double width, value;
   EmojiSection const *sections[] = {
     &self->recent,
     &self->people,
@@ -470,6 +483,8 @@ adj_value_changed (GtkAdjustment *adj, gpointer data)
   gsize i;
 
   /* Figure out which section the current scroll position is within */
+  width = gtk_widget_get_allocated_width (GTK_WIDGET (self));
+  value = gtk_adjustment_get_value (adj);
   for (i = 0; i < G_N_ELEMENTS (sections); ++i) {
     EmojiSection const *section = sections[i];
     GtkAllocation alloc;
@@ -479,8 +494,13 @@ adj_value_changed (GtkAdjustment *adj, gpointer data)
 
     gtk_widget_get_allocation (section->box, &alloc);
 
-    if (alloc.x == -1 || value < alloc.x - BOX_SPACE)
-      break;
+    if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL) {
+      if (alloc.x == -1 || value > alloc.x + alloc.width - width)
+        break;
+    } else {
+      if (alloc.x == -1 || value < alloc.x - BOX_SPACE)
+        break;
+    }
 
     select_section = section;
   }
@@ -489,7 +509,6 @@ adj_value_changed (GtkAdjustment *adj, gpointer data)
   for (i = 0; i < G_N_ELEMENTS (sections); ++i) {
     EmojiSection const *section = sections[i];
 
-
     /* TODO: scroll button into view */
     if (section == select_section) {
       GtkAdjustment *adj_sections;
@@ -682,7 +701,7 @@ pos_emoji_picker_class_init (PosEmojiPickerClass *klass)
                   G_TYPE_NONE, 0);
 
   gtk_widget_class_set_template_from_resource (widget_class,
-                                               "/sm/puri/phosh/osk-stub/ui/emoji-picker.ui");
+                                               "/mobi/phosh/osk-stub/ui/emoji-picker.ui");
 
   gtk_widget_class_bind_template_child (widget_class, PosEmojiPicker, scrolled_window);
   gtk_widget_class_bind_template_child (widget_class, PosEmojiPicker, scrolled_sections);
